/*
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

/* write little endian (for WAV) */
void write32le(int in, unsigned char * buf) {
	buf[0]=in&0xff;
	buf[1]=(in>>8)&0xff;
	buf[2]=(in>>16)&0xff;
	buf[3]=(in>>24)&0xff;
}

/* AFC decoder */

const short afccoef[16][2] =
{{0,0},
{0x0800,0},
{0,0x0800},
{0x0400,0x0400},
{0x1000,0xf800},
{0x0e00,0xfa00},
{0x0c00,0xfc00},
{0x1200,0xf600},
{0x1068,0xf738},
{0x12c0,0xf704},
{0x1400,0xf400},
{0x0800,0xf800},
{0x0400,0xfc00},
{0xfc00,0x0400},
{0xfc00,0},
{0xf800,0}};

int AFCdecodebuffer
(
    unsigned char			*input, // location of encoded source samples
    signed short         *out,   // location of destination buffer (16 bits / sample)
    short * histp,
	short * hist2p
)
{
int sample;
short nibbles[16];
int i,j;
char *src,*dst;
short idx;
short delta;
short hist=*histp;
short hist2=*hist2p;

	dst = (char*)out;

	src=input;
    delta = 1<<(((*src)>>4)&0xf);
	idx = (*src)&0xf;

	src++;
    
    for(i = 0; i < 16; i = i + 2) {
		j = ( *src & 255) >> 4;
		nibbles[i] = j;
		j = *src & 255 & 15;
		nibbles[i+1] = j;
		src++;
	}

     for(i = 0; i < 16; i = i + 1) {
		if(nibbles[i] >= 8) 
			nibbles[i] = nibbles[i] - 16;
     }
     
	 for(i = 0; i<16 ; i = i + 1) {

		sample = (delta * nibbles[i])<<11;
		sample += ((long)hist * afccoef[idx][0]) + ((long)hist2 * afccoef[idx][1]);
		sample = sample >> 11;

		if(sample > 32767) {
			sample = 32767;
		}
		if(sample < -32768) {
			sample = -32768;
		}

        *(short*)dst = (short)sample;
        dst = dst + 2;

		hist2 = hist;
        hist = (short)sample;
        
    }
	*histp=hist;
	*hist2p=hist2;

    return((int)src);
}

/* dump a WAV, decoding AFC */
/* return 0 on success, 1 on failure */
int dumpAFC(FILE * const infile, const int offset, const int size, const int srate, const char * const filename) {
	long oldpos;
	unsigned char inbuf[9];
	short outbuf[16];
	FILE * outfile;
	int sizeleft;
	int outsize,outsizetotal;
	short hist=0,hist2=0;
	
	unsigned char wavhead[44] = {
		0x52, 0x49, 0x46, 0x46, 0x00, 0x00, 0x00, 0x00,  0x57, 0x41, 0x56, 0x45, 0x66, 0x6D, 0x74, 0x20,
		0x10, 0x00, 0x00, 0x00, 0x01, 0x00, 0x01, 0x00,  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x02, 0x00, 0x10, 0x00, 0x64, 0x61, 0x74, 0x61,  0x00, 0x00, 0x00, 0x00
	};

	oldpos = ftell(infile);
	if (oldpos < 0) return 1;

	outfile = fopen(filename,"wb");
	if (!outfile) return 1;

	outsize = size/9*16*2;
	outsizetotal = outsize+8;
	write32le(outsizetotal,wavhead+4);
	write32le(outsize,wavhead+40);
	write32le(srate,wavhead+24);
	write32le(srate*2,wavhead+28);
	if (fwrite(wavhead,1,44,outfile)!=44) return 1;

	if (fseek(infile,offset,SEEK_SET)<0) return 1;

	for (sizeleft=size;sizeleft>=9;sizeleft-=9) {
		if (fread(inbuf,1,9,infile) != 9)
			return 1;

		AFCdecodebuffer(inbuf,outbuf,&hist,&hist2);

		if (fwrite(outbuf,1,16*2,outfile) != 16*2)
			return 1;
	}

	if (fclose(outfile)==EOF) return 1;

	if (fseek(infile,oldpos,SEEK_SET)<0) return 1;

	return 0;
}

int verbose = 0;

int main(int argc, char ** argv) {
	FILE * infile = NULL;
    long file_size;
    char out_name[]="dump_mono_0.wav";
    int i;

    if (argc != 2)
    {
        fprintf(stderr,"usage: %s blah.aw\n",argv[0]);
        exit(EXIT_FAILURE);
    }

    infile = fopen(argv[1], "rb");
    if (!infile)
    {
        perror("error opening input file");
        exit(EXIT_FAILURE);
    }

    fseek(infile,0,SEEK_END);
    file_size = ftell(infile);

    for (i = 0; i < 9; i++)
    {
        out_name[10] = '0' + i;
        dumpAFC(infile,i,file_size-i,32000,out_name);
        rewind(infile);
    }

    exit(EXIT_SUCCESS);
}
